#include "main.h"
#include "gpadpro.h"
#include "sidewind.h"
#include "psxpad.h"
#include "joy.h"

static int	mleft[2];
static int mright[2];
static int mup[2];
static int mdown[2];
static int centx[2];
static int centy[2];

static int NoJoys;
static int NoSidewinders;
static int NoPSX;
static int NoGamePadPros;

static int JoyMove=FALSE;

struct sSidewinder SideWinder;
struct sPSXPAD PSXPAD;

extern BYTE keys[NO_OF_KEYS];
extern SCANCODE sdef[NO_OF_KEYS];

//default joystick setup
JOYSETUP joydef[NO_JOY_CONFIGS]=
{
	{"j1but1","kp1butt1",-1},
	{"j1but2","kp1butt2",-1},
	{"j1but3","kp1butt3",-1},
	{"j1but4","knxtclist",-1},
	{"j1but5","knxtemu",-1},
	{"j1but6","",-1},
	{"j1but7","",-1},
	{"j1but8","",-1},
	{"j1select","kpcoin1",-1},
	{"j1start","kp1start",-1},
	{"j2but1","kp2butt1",-1},
	{"j2but2","kp2butt2",-1},
	{"j2but3","kp2butt3",-1},
	{"j2but4","klock",-1},
	{"j2but5","",-1},
	{"j2but6","",-1},
	{"j2but7","",-1},
	{"j2but8","",-1},
	{"j2select","kpcoin1",-1},
	{"j2start","kp2start",-1}
};

static int copycodes[NO_JOY_CONFIGS];

void setupjoyvals(void)
{
	int i;

	strcpy(pszTextValues[JOY_OFFSET],"None");
	for(i=0;i<NO_OF_KEYS;i++)
		strcpy(pszTextValues[JOY_OFFSET+i+1],sdef[i].szIdent);

	strcpy(pszTextValues[JOYNAME_OFFSET],"None");
	strcpy(pszTextValues[JOYNAME_OFFSET+JOY_2BUTTON],"2Button");
	strcpy(pszTextValues[JOYNAME_OFFSET+JOY_4BUTTON],"4Button");
	strcpy(pszTextValues[JOYNAME_OFFSET+JOY_6BUTTON],"6Button");
	strcpy(pszTextValues[JOYNAME_OFFSET+JOY_8BUTTON],"8Button");
	strcpy(pszTextValues[JOYNAME_OFFSET+JOY_SIDEWIND],"Sidewinder");
	strcpy(pszTextValues[JOYNAME_OFFSET+JOY_GAMEPADPRO],"GamePadPro");
	strcpy(pszTextValues[JOYNAME_OFFSET+JOY_PSXPAD],"PSX Pad");
}

int setupjoymenu(int n)
{
	int i;
	setarcconfig("Joy Type","",&JoyType,1,0,JOY_PSXPAD,n++,JOYNAME_OFFSET,CFG_INT);
	if(JoyType!=JOY_NONE)
	{
		for(i=0;i<NO_JOY_CONFIGS;i++)
		{
			copycodes[i]=joydef[i].idx;
			if(copycodes[i]==-1)
				copycodes[i]=JOY_OFFSET;
			else
				copycodes[i]+=(JOY_OFFSET+1);
			setarcconfig(joydef[i].szIdent,"",&copycodes[i],1,JOY_OFFSET,
							JOY_OFFSET+NO_OF_KEYS,n++,0,CFG_INT);
		}
	}
	return n;
}

void updatejoys(void)
{
	int i;
	for(i=0;i<NO_JOY_CONFIGS;i++)
	{
		joydef[i].idx=(copycodes[i]==JOY_OFFSET)?-1:copycodes[i]-(JOY_OFFSET+1);

		if(joydef[i].idx>0)
			strcpy(joydef[i].szName,sdef[joydef[i].idx].szIdent);
	}
}


void getjoysticksetup(void)
{
	int i;
	char szWork[16];
	for(i=0;i<NO_JOY_CONFIGS;i++)
		getjoyidx(&joydef[i]);
}

void getjoyidx(pJOYSETUP pJoySetup)
{
	int i;
	int idx=-1;
	for(i=0;i<NO_OF_KEYS;i++)
	{
		if(!stricmp(pJoySetup->szName,sdef[i].szIdent))
		{
			idx=i;
			break;
		}
	}
	pJoySetup->idx=idx;
}

void checkcontroller(void)
{
	switch(JoyType)
	{
		case	JOY_2BUTTON:
		case	JOY_4BUTTON:
		case	JOY_6BUTTON:
		case	JOY_8BUTTON:
			NoJoys=checkjoy();
			if(NoJoys<1)
				JoyType=0;
			else
				calibratejoy();
			break;
		case	JOY_SIDEWIND:
			NoSidewinders=GetSidewinderStatus(&SideWinder);
				// Don't Reset. Sidewinder may not respond first try.
			break;
		case	JOY_GAMEPADPRO:
			NoGamePadPros=checkgpp();
			if(NoGamePadPros<1)
				JoyType=0;
			break;
		case	JOY_PSXPAD:
			NoPSX=psx1_init();
			if(NoPSX<1)
				JoyType=0;
			break;
		default:
			loginfo(1,"Illegal Joystick Value (%d), Joystick disabled",JoyType);
			JoyType=0;
	}
}

void calibratejoy(void)
{
	int dummy,nojoys,i;
	int minx,miny,maxx,maxy;
	int workx,worky,workx2,worky2;

	centx[0]=centy[0]=centx[1]=centy[1]=0;
	nojoys=(JoyType==JOY_6BUTTON || JoyType==JOY_8BUTTON)?2:NoJoys;

//get centre position
	for(i=0;i<8;i++)
	{
		readjoystick(nojoys,&dummy,&workx,&worky,&workx2,&worky2);
		centx[0]+=workx;
		centy[0]+=worky;
		centx[1]+=workx2;
		centy[1]+=worky2;
	}

	centx[0]>>=3;
	centy[0]>>=3;
	centx[1]>>=3;
	centy[1]>>=3;


	for(i=0;i<nojoys;i++)
	{
//make up min max values
		minx=centx[i]-(centx[i]>>1);
		miny=centy[i]-(centy[i]>>1);
		maxx=centx[i]+(centx[i]>>1);
		maxy=centy[i]+(centy[i]>>1);
//make up some believable deltas
		mleft[i]=minx+((centx[i]-minx)/8);
		mright[i]=maxx-((maxx-centx[i])/8);
		mup[i]=miny+((centy[i]-miny)/8);
		mdown[i]=maxy-((maxy-centy[i])/8);
	}
}

int handlejoystick(void)
{
	JoyMove=FALSE;

	switch(JoyType)
	{
		case JOY_2BUTTON:
		case JOY_4BUTTON:
		case JOY_6BUTTON:
		case JOY_8BUTTON:
			readstandardjoy();
			break;
		case JOY_SIDEWIND:
			readsidewinder();
			break;
		case JOY_GAMEPADPRO:
			readgamepadpro();
			break;
		case JOY_PSXPAD:
			readpsxpad();
	}

	return JoyMove;
}

void readstandardjoy(void)
{
	int i,joybuttons,n,nbut,bmask,bidx,p2;
	int stdidx;
	int injoyx[2];
	int injoyy[2];
	readjoystick(NoJoys,&joybuttons,&injoyx[0],&injoyy[0],&injoyx[1],&injoyy[1]);

	bmask=16;

	if(JoyType==JOY_2BUTTON)
	{
//2 possible joysticks
		stdidx=0;
		for(i=0;i<NoJoys;i++)
		{
			p2=i*10;
			for(bidx=0;bidx<2;bidx++,bmask<<=1,stdidx++)
			{
				if(!(joybuttons&bmask))
				{
					SETJOYBUTTON(bidx+p2)
					JOYMOVE
				}
			}
		}
	} else
	{
//too many buttons - so there can only be one joystick
		for(n=0;n<4;n++)
		{
			if(!(joybuttons&bmask))
			{
				SETJOYBUTTON(n)
				JOYMOVE
			}
			bmask<<=1;
		}
	}

//six button, 2 buttons are second x/y axis
	if(JoyType==JOY_6BUTTON)
	{

		if(injoyx[1]==(centx[1]-128))
		{
			SETJOYBUTTON(4)
			JOYMOVE
		}
		if(injoyy[1]==(centy[1]-128))
		{
			SETJOYBUTTON(5)
			JOYMOVE
		}
	}
//8button, 4 buttons mapped to second x/y axis
	if(JoyType==JOY_8BUTTON)
	{
		if(injoyx[1]<mleft[1])
		{
			SETJOYBUTTON(4)
			JOYMOVE
		}

		if(injoyy[1]<mup[1])
		{
			SETJOYBUTTON(5)
			JOYMOVE
		}

		if(injoyx[1]>mright[1])
		{
			SETJOYBUTTON(6)
			JOYMOVE
		}

		if(injoyy[1]>mdown[1])
		{
			SETJOYBUTTON(7)
			JOYMOVE
		}
	}

//copy across joystick movement
	for(i=0;i<NoJoys;i++)
	{
		if(injoyy[i]<mup[i])
		{
			keys[K_UP+KEY_P2]=1;
			JOYMOVE
		}
		if(injoyy[i]>mdown[i])
		{
			keys[K_DOWN+KEY_P2]=1;
			JOYMOVE
		}
		if(injoyx[i]<mleft[i])
		{
			keys[K_LEFT+KEY_P2]=1;
			JOYMOVE
		}
		if(injoyx[i]>mright[i])
		{
			keys[K_RIGHT+KEY_P2]=1;
			JOYMOVE
		}
	}
}

void readsidewinder(void)
{
	int i,nojoys,bidx;
	static unsigned long int swdidx[4]={SW_UP,SW_DOWN,SW_LEFT,SW_RIGHT};
	static unsigned long int swbidx[10]={SW_A,SW_B,SW_C,SW_X,SW_Y,SW_Z,
										SW_LEFTFINGER,SW_RIGHTFINGER,SW_M,SW_START};

//get the details
	nojoys=GetSidewinderStatus(&SideWinder);
	if(nojoys<1)
		return;

	if(nojoys>2)
		nojoys=2;


	for(i=0;i<nojoys;i++)
		setgenjoy(SideWinder.dwSidewinder[i],i,swdidx,swbidx);
}


void readgamepadpro(void)
{

	int i,nojoys,bidx;
	unsigned long int inval;
	static unsigned long int gpdidx[4]={GPP_UP,GPP_DOWN,GPP_LEFT,GPP_RIGHT};
	static unsigned long int gpbidx[10]={GPP_RED,GPP_YELLOW,GPP_GREEN,GPP_BLUE,
								GPP_L1,GPP_R1,GPP_L2,GPP_R2,GPP_SELECT,GPP_START};

	nojoys=NoGamePadPros;

	for(i=0;i<nojoys;i++)
	{
		inval=readgpp(i);
		if(inval==1)
			return;
		setgenjoy(inval,i,gpdidx,gpbidx);
	}
}

void readpsxpad(void)
{
	int i,nojoys,p2,bidx;
	int ret=FALSE;
	static unsigned long int pdidx[4]={PSX_UP,PSX_DOWN,PSX_LEFT,PSX_RIGHT};
	static unsigned long int pbidx[10]={PSX_CROSS,PSX_CIRCLE,PSX_TRIANGLE,PSX_SQUARE,
									PSX_L1,PSX_R1,PSX_L2,PSX_R2,PSX_SELECT,PSX_START};

//get the details
	nojoys=psx1_poll(&PSXPAD);
	if(nojoys<1)
		return;

	if(nojoys>2)
		nojoys=2;
	for(i=0;i<nojoys;i++)
		setgenjoy(PSXPAD.dwPSXPAD[i],i,pdidx,pbidx);
}

void setgenjoy(unsigned long int inval,int joyidx,unsigned long int *dirs,unsigned long int *buts)
{
	int i;
	int p2dir=JOY2_DIR;
	int p2but=JOY2_BUT;


//directions
	for(i=0;i<4;i++)
	{
		if(inval&dirs[i])
		{
			keys[i+p2dir]=128;
			JOYMOVE
		}
	}
//buttons
	for(i=0;i<10;i++)
	{
		if(inval&buts[i])
		{
			SETJOYBUTTON(i+p2but)
			JOYMOVE
		}

	}
}

void movement(int *value,int page)
{
	int i;
	for(i=0;i<2;i++)
	{
		if(keys[K_DOWN+KEY_P2])
		{
			*value+=1;
			checkwhichplayer(K_DOWN);
		}
		if(keys[K_UP+KEY_P2])
		{
			checkwhichplayer(K_UP);
			*value-=1;
		}

		if(keys[K_LEFT+KEY_P2])
		{
			checkwhichplayer(K_LEFT);
			*value-=page;
		}
		if(keys[K_RIGHT+KEY_P2])
		{
			checkwhichplayer(K_RIGHT);
			*value+=page;
		}
	}
}

void checkwhichplayer(int idx)
{
	if(!noCocktailMode)
		whichplayer(keys[idx]==0);
}

void whichplayer(int idx)
{
	static clock_t lastflip=0;
	clock_t now;
	clock_t tickcount;
	int seconds;

	int flipOK=FALSE;

	if(idx!=PlayerControl)
	{
//see if we've flipped yet
		if(!lastflip||!FlipDelay)
		{
//nope, so flip away
			flipOK=TRUE;
		} else
		{
//we have, so check the delay's OK - to avoid constant flipping
			now=clock();
			tickcount=now-lastflip;
			seconds=tickcount/clocks_per_second;
			if(seconds>=FlipDelay)
				flipOK=TRUE;
		}
//check if we can flip
		if(flipOK)
		{
			noflipx=!noflipx;
			noflipy=!noflipy;
			PlayerControl=idx;
			lastflip=clock();
		}
	}
}


int startpressed(void)
{
	return PlayerControl?keys[K_2_B1]:keys[K_START1];
}


void setstart(void)
{
	if(PlayerControl)
		keys[K_2_B1]=1;
	else
		keys[K_START1]=1;
}

int p2button1(void)
{
	return PlayerControl?0:keys[K_2_B1];
}
