//
// Written by: Robert C. Pendleton
//
// Copyright 1993 by Robert C. Pendleton, all right reserved
//
// Non-commercial use by individuals is permitted.
//
//

#ifndef _FIXED_H_
#define _FIXED_H_

//----------------------------------------------
//
// handy macros
//

#define max(a,b) (((a) > (b)) ? (a) : (b))
#define min(a,b) (((a) < (b)) ? (a) : (b))
//#define abs(a) (((a)<0) ? -(a) : (a))
//#define sign(a) (((a)<0) ? -1 : (a)>0 ? 1 : 0)


//----------------------------------------------
//
// fixed point math
//

typedef long fp14;

//----------------------------------------------
//
// handy fixed point constants
//

#define fpZero int2fp(0)
#define fpOne int2fp(1)
#define fpTwo int2fp(2)
#define fpHalf (1 << 13)

//----------------------------------------------
//
// Divide two 18.14 fixed point numbers giving
// a 18.14 fixed point result.
//

fp14 fpDiv(fp14 d1, fp14 d2);
//----------------------------------------------
//
// Fixed point inverse, need to find a better way.
//


//----------------------------------------------
//
// Divide two 18.14 fixed point numbers giving
// a 18.14 fixed point result.
//

fp14 fpMul(fp14 m1, fp14 m2);
//----------------------------------------------
//
// Convert an integer to n N.14 fixed point
// number and back.
//

#define int2fp(x) ((x) << 14)
#define fp2int(x) ((x) >> 14)

//----------------------------------------------
//
// Get the fractional part of an N.14 fixed point
// number.

#define fpFract(x) ((x) & 0x00003fff)

//----------------------------------------------
//
// Convert a N.14 fixed point number to a
// double and back. Handy for printing and for
// those times when you really do want to work
// with floating point values.
//

#define fp2float(x) (((double)(x)) / (1 << 14))
#define float2fp(x) ((fp14) (((double)x) * (1 << 14)))

//----------------------------------------------
//
// Table based fixed point sine and cosine
// functions.
//

extern fp14 sine[];
extern fp14 cosine[];
#define fpInv(x)    fpDiv(fpOne, x)
#define fpSin(x) (sine[(x) & 0x3ff])
#define fpCos(x) (cosine[(x) & 0x3ff])

int adjust(fp14 val);

#endif
