#include "main.h"
#include "glist.h"
#include "disp.h"
#include "info.h"
#include "launch.h"
#include "lst.h"

// LasVegas - Boosted buffers for larger game lists.
const long	GAME_SIZE=262144L;
const long	CLONE_SIZE= 131072L;
const long	NEO_SIZE= 131072L;


static CONTROLDESC controldescs[]=
{
	{"joy2way",&filtjoy2way[0]},
	{"joy4way",&filtjoy4way[0]},
	{"joy8way",&filtjoy8way[0]},
	{"doublejoy2way",&filtdoublejoy2way[0]},
	{"doublejoy4way",&filtdoublejoy4way[0]},
	{"doublejoy8way",&filtdoublejoy8way[0]},
	{"paddle",&filtpaddle[0]},
	{"dial",&filtdial[0]},
	{"trackball",&filttrackball[0]},
	{"stick",&filtstick[0]},
	{0,0}
};

void getexeinfo(char *szName,char *szOut)
{
	FILE	*fExe;
	struct ftime ft;
	fExe=fopen(szName,"rb");
	if(fExe)
	{
		getftime(fileno(fExe),&ft);
		fclose(fExe);
		sprintf(szOut,"%u:%u:%u::%u/%u/%u|",ft.ft_hour, ft.ft_min,ft.ft_tsec * 2,
						 ft.ft_month,ft.ft_day,ft.ft_year+1980);
	}
	else
		*szOut=0;
}


int getnamelist(char *pNeoList,char *szFile,long maxsize)
{
	int	nRet=FALSE;
	int	n;
	int	bSimpFile=FALSE;
	char	*pWork;
	char	*pWork2;
	char	szWork[256];
	long	lCount=0;
	FILE	*fIn;
	size_t	st;
	static char	buffer[1024];
	static char	szTemp[1024];
	static char	szCommand[1024];
	SOUND_UPDATE_INIT

	*pNeoList=0;
//check for single game, just return true with a blank list
	if(*Emulators[CurEmu].szSingle)
		return TRUE;

	if(CurEmu==EMU_MAME)
	{
//check for optional mame.nms
		if(checkfile("mame.nms"))
		{
			strcpy(szFile,"mame.nms");
			bSimpFile=TRUE;
		} else
			bSimpFile=FALSE;
	} else
		bSimpFile=TRUE;

	if(bSimpFile)
	{

		fIn=fopen(szFile,"rt");
		if(fIn)
		{
			while(TRUE)
			{
				if(!tidyget(szCommand,128,fIn))
					break;
				getitem(szTemp,szCommand,1,127,'|');
				strupr(szTemp);
				sprintf(buffer,"^%s{%s{",szTemp,getitem(szWork,szCommand,0,127,'|'));
				lCount+=strlen(buffer);
				if(lCount>=GAME_SIZE)
					break;

				strcpy(pNeoList,buffer);
				pNeoList+=strlen(buffer);
				SOUND_UPDATE_TRIP(10)
			}
			fclose(fIn);
			nRet=TRUE;
		}
		return nRet;
	}

//does the file exist ?
	if(!checkfile(szFile) || !checkfile("mameinfo.lst"))
	{
//nope, so make it
		if(!makelists())
			return -1;
	}


	fIn=fopen(szFile,"rb");
	if(!fIn)
		return nRet;
//get the date in
	fread(buffer,1,25,fIn);
	pWork=buffer;
	pWork2=buffer+1;

	n=0;
	while(*pWork!='|'&&n<25)
	{
		pWork++;
		n++;
	}
	fclose(fIn);

	if(*pWork=='|')
	{
		pWork++;
		*pWork=0;
		sprintf(szWork,"%s%s",Emulators[CurEmu].szPath,Emulators[CurEmu].szInfoExe);
		getexeinfo(szWork,szTemp);
		if(*buffer!=INFO_IDENT_CHAR||strcmp(pWork2,szTemp))
		{
			if(!makelists())
				return -1;
		}
	} else
		return nRet;

	fIn=fopen(szFile,"rb");
	if(fIn)
	{
		st=fread(pNeoList,1,(size_t)maxsize,fIn);
		pNeoList[(int)st]=0;
		nRet=TRUE;
	}
	fclose(fIn);
	return	nRet;
}

int isneogeo(char *szName,char *pNeoList)
{
	char	szWork[64];
	if(!*pNeoList)
		return  FALSE;
	sprintf(szWork,"^%s{",szName);
	if(strstr(pNeoList,szWork))
		return TRUE;
	return FALSE;
}

//creates a file of unique games + configs
//and copies it into memory
int createlist(int *pNoGames,int nDispRot,int lastgame)
{
	pGETFILE	pGetFiles;
	char	*pGameText=NULL;
	char	*pCloneText=NULL;
	char	*pNeoText=NULL;
	int	bQuick,nret;
	char	szRMS[64];
	static char szUpdate[512];

	int n;
//set game cursor
	CurPos=0;
	bQuick=FALSE;

	if(!noInstantStart||ReRun)
	{
		getrmsfile(szRMS);

		if(checkfile(szRMS))
			bQuick=TRUE;
	}

	if(bQuick)
	{
		*pNoGames=quickread(szRMS);
	} else
	{
		pGetFiles=myalloc(FILESIZE(MAX_GAMES));
		if(!pGetFiles)
		{
			printf("Failed to Allocate Memory for Game File List\n");
			return -1;
		}
		pGameText=myalloc(GAME_SIZE);
		if(!pGameText)
		{
			printf("Failed to Allocate Memory for Long Game Names\n");
			myfree(pGetFiles);
			return -1;
		}
		pCloneText=myalloc(CLONE_SIZE);
		if(!pCloneText)
		{
			printf("Failed to Allocate Memory for Clone Game Names\n");
			myfree(pGetFiles);
			myfree(pGameText);
			return -1;
		}
		pNeoText=myalloc(NEO_SIZE);
		if(!pNeoText)
		{
			printf("Failed to Allocate Memory for NeoGeo Game Names\n");
			myfree(pCloneText);
			myfree(pGetFiles);
			myfree(pGameText);
			return -1;
		}
		*pCloneText=0;
		*pGameText=0;
		*pNeoText=0;

		emudir(NULL);

		*pNoGames=makegamelist(pGetFiles,pNoGames,MAX_GAMES);
		menudir();

		nret=getnamelist(pGameText,CurEmu==EMU_MAME?
			MAME_LIST:Emulators[CurEmu].szTableName,GAME_SIZE);

		if(nret<=0 && CurEmu!=EMU_MAME && checkfile(Emulators[CurEmu].szUpdateExe))
		{
			sprintf(szUpdate,"%s %s %s",Emulators[CurEmu].szUpdateExe,Emulators[CurEmu].szPath,Emulators[CurEmu].szRomsDir);
			runcommand(szUpdate,FALSE,FALSE,NULL,NULL,FALSE,NULL);
			nret=getnamelist(pGameText,MAME_LIST,GAME_SIZE);
		}


		if(nret<=0&&!Emulators[CurEmu].szPattern)
		{
			if(nret==FALSE)
			{
				if(CurEmu==EMU_MAME)
					printf("Unable to get game list from MAME\n"
						"(command: 'MAME -listfull' failed)\n"
						"Ensure your using DOS MAME and ArcadeOS.CFG is setup correctly\n");
				else
					printf("Unable to open .NMS file for %s\n",
						Emulators[CurEmu].szEmuName);
			}
			myfree(pGetFiles);
			myfree(pGameText);
			myfree(pCloneText);
			myfree(pNeoText);
			return -2;
		}

		if(CurEmu==EMU_MAME)
			getnamelist(pCloneText,CLONE_LIST,CLONE_SIZE);
		else if(Emulators[CurEmu].bSupportsMerged)
			getnamelist(pCloneText,Emulators[CurEmu].szCloneFile,CLONE_SIZE);

		if(CurEmu==EMU_MAME)
			getnamelist(pNeoText,NEOGEO_LIST,NEO_SIZE);

		*pNoGames=addlongnames(pGetFiles,*pNoGames,pGameText,pCloneText);

		*pNoGames=makermsfile(pGetFiles,*pNoGames,pNeoText);

		myfree(pNeoText);
		myfree(pGetFiles);
		myfree(pGameText);
		myfree(pCloneText);
	}

	setupgameinfo();
	readgamelist(nDispRot);

	if(lastgame>=Emulators[CurEmu].nGameTotals[CUSTOM_LIST][ROTATION])
		lastgame=0;

	if(lastgame<0)
		lastgame=0;

	CurPos=lastgame;
	if(*pNoGames<=0)
		OffSet[CurEmu]=0;

	return *pNoGames;
}

int addlongnames(pGETFILE pGetFiles,int nGames,char *pGameText,char *pCloneText)
{
	int	n;
	char *pClones;
	GETFILE	getfile;
	int	nRet;
	SOUND_UPDATE_INIT

	nRet=nGames;



//check for a single game emu/app
	if(*Emulators[CurEmu].szSingle)
	{
		*pGetFiles[0].FileName=0;
		pGetFiles[0].bGotName=TRUE;
		safecopy(pGetFiles[0].szLongName,Emulators[CurEmu].szSingle,63);
		nRet=1;
	} else
	{
		for(n=0;n<nGames;n++)
		{
			SOUND_UPDATE_TRIP(10)

			getrealname(&pGetFiles[n],pGameText);

			if(!noMergedROMs[CurEmu]&&Emulators[CurEmu].bSupportsMerged)
			{
				pClones=pCloneText;
				while(pClones)
				{
					pClones=getclones(pGetFiles[n].FileName,pClones,getfile.FileName);

					if(pClones && !dupfilesort(getfile.FileName,pGetFiles))
					{
						getrealname(&getfile,pGameText);
						if((getfile.bGotName || !noUnknowns)&&(nRet<MAX_GAMES))
						{
							getfile.bType=pGetFiles[n].bType|128;
							memcpy(&pGetFiles[nRet++],&getfile,sizeof(GETFILE));
						}
					}
				}
			}
		}
	}
	qsort((void *)pGetFiles,nRet,sizeof(GETFILE),longsort);
	return nRet;
}

int longsort(const void *a, const void *b)
{
	pGETFILE pa;
	pGETFILE pb;

	pa=(pGETFILE)a;
	pb=(pGETFILE)b;

	return strcmpi(pa->szLongName,pb->szLongName);
}

void writegamelist(int nGames)
{
	FILE	*fin;
	int	pos;
	char buffer[512];
	pMAMEGAME pLocal=pMameGames;

	SOUND_UPDATE_INIT

	if(!nGames)
		return;
	getrmsfile(buffer);


	fin=fopen(buffer,"r+b");

	if(!fin)
		return;
	loginfo(1,"write game list to file");

	pos=0;
	while(TRUE)
	{
		if(pLocal->bChanged)
		{
			pLocal->bChanged=FALSE;
			if((pLocal->SaveOrient&7)!=(pLocal->Orientation&7))
			{
				switch(pLocal->Orientation&7)
				{
					case	0:	//move to normal
						Emulators[CurEmu].nGameTotals[CUSTOM_LIST][1]+=1;
						switch(pLocal->SaveOrient&7)
						{
							case	1:
							case	2:
							case	3:
								Emulators[CurEmu].nGameTotals[CUSTOM_LIST][2]-=1;
								break;
							case	4:
								Emulators[CurEmu].nGameTotals[CUSTOM_LIST][3]-=1;
						}
						break;
					case 1:	//move to
					case 2:	//rotated left/right or
					case 3:	//not rotated
						switch(pLocal->SaveOrient&7)
						{
							case	0:	//Normal
								Emulators[CurEmu].nGameTotals[CUSTOM_LIST][1]-=1;
								Emulators[CurEmu].nGameTotals[CUSTOM_LIST][2]+=1;
								break;
							case	1:
							case	2:	//totals stay the say
							case	3:
								break;
							case	4:	//Disabled
								Emulators[CurEmu].nGameTotals[CUSTOM_LIST][3]-=1;
								Emulators[CurEmu].nGameTotals[CUSTOM_LIST][2]+=1;
						}
						break;
					case 4:	//move to disabled
						Emulators[CurEmu].nGameTotals[CUSTOM_LIST][3]+=1;
						switch(pLocal->SaveOrient&7)
						{
							case	0:	//Normal
								Emulators[CurEmu].nGameTotals[CUSTOM_LIST][1]-=1;
								break;
							case	1:
							case	2:	//rotated
							case	3:
								Emulators[CurEmu].nGameTotals[CUSTOM_LIST][2]-=1;
						}
				}
			}
			pLocal->SaveOrient=pLocal->Orientation;
			if(!fsetpos(fin,&pLocal->fFilePos))
				fwrite(pLocal,sizeof(MAMEGAME),1,fin);
			SOUND_UPDATE_TRIP(10)
		}

		pLocal++;
		pos++;
		  if(pos>=nGames||pos>=MAX_GAMES)
			break;
	}
	fclose(fin);
}

void readgamelist(int nDispRot)
{
	FILE	*fin;
	int	nout=0;
	int	bWant;
	int	Orient;
	int	Cust;

	static char	buffer[512];
	pMAMEGAME	pLocal=pMameGames;
	MAMEDESC	mamedesc;
	MAMEGAME	Game;
	SOUND_UPDATE_INIT

	getrmsfile(buffer);

	fin=fopen(buffer,"rb");

	if(!fin)
		return;
	loginfo(1,"Read game list from disk");

//skip past header
	fseek(fin,RMS_HEAD_SIZE,SEEK_SET);

	while(TRUE)
	{
		if(!fread(&Game,1,sizeof(MAMEGAME),fin))
			break;
		loginfo(1,"Read in info for %s",Game.GameName);

		bWant=FALSE;

		if(!noFiltersActive&&CurEmu==EMU_MAME)
		{
			fillinfo(Game.GameName,&mamedesc);
			loginfo(1,"Game details copied for %s",mamedesc.szFileName);
		}
		Orient=Game.Orientation&7;
		Cust=(Game.Orientation>>4)%NO_CUSTOM_LISTS;

		if(Cust==CUSTOM_LIST||!noCustomContainsAll[CurEmu])
		{
			switch(nDispRot)
			{
				case	0:
					bWant=TRUE;
					break;
				case	1:	//normal
					if(Orient==0)
						bWant=TRUE;
					break;
				case	2:	//rotated
					if(Orient==1||Orient==2||Orient==3)
						bWant=TRUE;
					break;
				case  3: //Disabled
					if(Orient==4)
						bWant=TRUE;
					break;
			}
		}
		if(!noFiltersActive&&CurEmu==EMU_MAME)
		{
			loginfo(1,"Check Filter for %s",mamedesc.szFileName);
			checkfilter(&mamedesc,&bWant);

//LasVegas - check filter for rom folders
			if(filtrom[CUSTOM_LIST] && Game.bRomFolder!=filtrom[CUSTOM_LIST])
			{
			  loginfo(1,"ROM Folder Filter failed for %s, game=%d,filt=%d",
					mamedesc.szFileName,Game.bRomFolder,filtrom[CUSTOM_LIST]);
			  bWant=FALSE;
			}
		}
		if(bWant)
		{
			Game.bChanged=FALSE;
			Game.SaveOrient=Game.Orientation;
			memcpy(pLocal,&Game,sizeof(MAMEGAME));
//clear 'game picked by random' byte
			if(!ReRun)
				pLocal->bRandomRun=FALSE;
			pLocal++;
			nout++;
			if(nout>=MAX_GAMES)
				break;
		}
		SOUND_UPDATE_TRIP(10)
	}
	fclose(fin);
//update total
	Emulators[CurEmu].nGameTotals[CUSTOM_LIST][nDispRot]=nout;
}

void checkfilter(pMAMEDESC pMameDesc,int *pWant)
{
	int x,y;

//players
	if(pMameDesc->nPlayers<minfiltplayers[CUSTOM_LIST])
	{
		loginfo(1,"Min.No.Players Filter failed for %s, game=%d,filt=%d",
				pMameDesc->szFileName,pMameDesc->nPlayers,minfiltplayers[CUSTOM_LIST]);
		FILTER_FAIL
	}

	if(pMameDesc->nPlayers>maxfiltplayers[CUSTOM_LIST])
	{
		loginfo(1,"Max.No.Players Filter failed for %s, game=%d,filt=%d",
				pMameDesc->szFileName,pMameDesc->nPlayers,maxfiltplayers[CUSTOM_LIST]);
		FILTER_FAIL
	}

//year
	if(*pMameDesc->szYear)
	{
		if(atoi(pMameDesc->szYear)<minfiltyear[CUSTOM_LIST])
		{
			loginfo(1,"Min. Year Filter failed for %s, game=%d,filt=%d",
				pMameDesc->szFileName,pMameDesc->szYear,minfiltyear[CUSTOM_LIST]);
			FILTER_FAIL
		}
		if(atoi(pMameDesc->szYear)>maxfiltyear[CUSTOM_LIST])
		{
			loginfo(1,"Max. Year Filter failed for %s, game=%d,filt=%d",
				pMameDesc->szFileName,pMameDesc->szYear,maxfiltyear[CUSTOM_LIST]);
			FILTER_FAIL
		}
	}

//frequency
	if(pMameDesc->nFreq<minfiltfreq[CUSTOM_LIST])
	{
		loginfo(1,"Min. Freq. Filter failed for %s, game=%d,filt=%d",
			pMameDesc->szFileName,pMameDesc->nFreq,minfiltfreq[CUSTOM_LIST]);
		FILTER_FAIL
	}
	if(pMameDesc->nFreq>maxfiltfreq[CUSTOM_LIST])
	{
		loginfo(1,"Max. Freq. Filter failed for %s, game=%d,filt=%d",
			pMameDesc->szFileName,pMameDesc->nFreq,maxfiltfreq[CUSTOM_LIST]);
		FILTER_FAIL
	}
//coin slots
	if(pMameDesc->nCoins<minfiltcoin[CUSTOM_LIST])
	{
		loginfo(1,"Min. Coin Filter failed for %s, game=%d,filt=%d",
			pMameDesc->szFileName,pMameDesc->nCoins,minfiltcoin[CUSTOM_LIST]);
		FILTER_FAIL
	}
	if(pMameDesc->nCoins>maxfiltcoin[CUSTOM_LIST])
	{
		loginfo(1,"Max. Coin Filter failed for %s, game=%d,filt=%d",
			pMameDesc->szFileName,pMameDesc->nCoins,maxfiltcoin[CUSTOM_LIST]);
		FILTER_FAIL
	}
//resolution
	if(*pMameDesc->szResolution!='0')
	{
		getres(pMameDesc->szResolution,&x,&y);
		if(x<minfiltx[CUSTOM_LIST])
		{
			loginfo(1,"Min X Res Filter failed for %s, game=%d,filt=%d",
				pMameDesc->szFileName,x,minfiltx[CUSTOM_LIST]);
			FILTER_FAIL
		}
		if(x>maxfiltx[CUSTOM_LIST])
		{
			loginfo(1,"Max X Res Filter failed for %s, game=%d,filt=%d",
				pMameDesc->szFileName,x,maxfiltx[CUSTOM_LIST]);
			FILTER_FAIL
		}
		if(y<minfilty[CUSTOM_LIST])
		{
			loginfo(1,"Min Y Res Filter failed for %s, game=%d,filt=%d",
				pMameDesc->szFileName,y,minfilty[CUSTOM_LIST]);
			FILTER_FAIL
		}
		if(y>maxfilty[CUSTOM_LIST])
		{
			loginfo(1,"Max Y Res Filter failed for %s, game=%d,filt=%d",
				pMameDesc->szFileName,y,maxfilty[CUSTOM_LIST]);
			FILTER_FAIL
		}
	}
//control
	if(checkcontrolfilter(pMameDesc))
	{
		loginfo(1,"Control Filter failed for %s",pMameDesc->szFileName);
		FILTER_FAIL
	}
//buttons
	if(pMameDesc->nButtons<minfiltbuttons[CUSTOM_LIST])
	{
		loginfo(1,"Min. Buttons Filter failed for %s, game=%d,filt=%d",
			pMameDesc->szFileName,pMameDesc->nButtons,minfiltbuttons[CUSTOM_LIST]);
		FILTER_FAIL
	}
	if(pMameDesc->nButtons>maxfiltbuttons[CUSTOM_LIST])
	{
		loginfo(1,"Max. Buttons Filter failed for %s, game=%d,filt=%d",
			pMameDesc->szFileName,pMameDesc->nButtons,maxfiltbuttons[CUSTOM_LIST]);
		FILTER_FAIL
	}
//horizontal (inverted logic)
	if(filthorz[CUSTOM_LIST] && pMameDesc->nHorizontal)
	{
		loginfo(1,"Horz Filter failed for %s, game=%d,filt=%d",
			pMameDesc->szFileName,!pMameDesc->nHorizontal,filthorz[CUSTOM_LIST]);
		FILTER_FAIL
	}
//vertical (inverted logic)
	if(filtvert[CUSTOM_LIST] && !pMameDesc->nHorizontal)
	{
		loginfo(1,"Vert Filter failed for %s, game=%d,filt=%d",
			pMameDesc->szFileName,pMameDesc->nHorizontal,filtvert[CUSTOM_LIST]);
		FILTER_FAIL
	}
//vector (inverted logic)
	if(filtvector[CUSTOM_LIST] && *pMameDesc->szResolution=='0')
	{
		loginfo(1,"Max. Year Filter failed for %s, game=%d,filt=%d",
			pMameDesc->szFileName,*pMameDesc->szResolution!='0',filtvector[CUSTOM_LIST]);
		FILTER_FAIL
	}
//raster (inverted logic)
	if(filtraster[CUSTOM_LIST] && *pMameDesc->szResolution!='0')
	{
		loginfo(1,"Max. Year Filter failed for %s, game=%d,filt=%d",
			pMameDesc->szFileName,*pMameDesc->szResolution=='0',filtraster[CUSTOM_LIST]);
		FILTER_FAIL
	}

}

int checkcontrolfilter(pMAMEDESC pMameDesc)
{
	int idx=0;
	int custoffset;
	int *pWork;
	custoffset=CUSTOM_LIST;

	while(controldescs[idx].pIdent)
	{
//get the value
		pWork=controldescs[idx].pValue;
//custom list offset
		pWork+=custoffset;
//is it off (inverted)? if so, if the string matches - we fail

		if(*pWork && !strcmpi(controldescs[idx].pIdent,pMameDesc->szControl))
			return TRUE;
		idx++;
	}
	return FALSE;
}
 /*
	char  szFileName[256];
	char  szDescription[256];
	char  szCloneOf[256];
	char  szResolution[256];
	char  szControl[256];
	char  szYear[256];
	char  szMan[256];
	int	nColors;
	int	nFreq;
	int	nRotated;
	int	nOkay;
	int	nClone;
	int	nVector;
	int	nPlayers;
	int	nCoins;
	int	nColorDepth;
	int	nHiScoreOK;
	int	nColorOK;
*/
int quickread(char *szName)
{
	pMAMEGAME	pChecks;
	FILE	*fRMS;
	long	MaxChecks;
	unsigned long lsize;
	unsigned int noread,lcount;
	int	nRet=0;
	int	CList;

	memset(Emulators[CurEmu].nGameTotals,0,sizeof(int)*NO_CUSTOM_LISTS*4);

//make the biggest buffer we can for our check file
	MaxChecks=3000;
	lsize=(unsigned long)((unsigned long)MaxChecks*(unsigned long)sizeof(MAMEGAME));
	pChecks=myalloc(lsize);

	if(!pChecks)
		return nRet;

	fRMS=fopen(szName,"rb");
	fseek(fRMS,RMS_HEAD_SIZE,SEEK_SET);
//buffer the existing file

	if(fRMS)
	{
		while(TRUE)
		{
			noread=fread(pChecks,sizeof(MAMEGAME),(size_t)MaxChecks,fRMS);
			if((int)noread<=0)
				break;
			nRet+=(int)noread;
			for(lcount=0;lcount<noread;lcount++)
			{

				CList=(pChecks[lcount].Orientation>>4)%NO_CUSTOM_LISTS;
				Emulators[CurEmu].nGameTotals[CList][0]+=1;
				switch(pChecks[lcount].Orientation&7)
				{
					case	0:
						Emulators[CurEmu].nGameTotals[CList][1]+=1;
						break;
					case	1:
					case	2:
					case	3:
						Emulators[CurEmu].nGameTotals[CList][2]+=1;
						break;
					case	4:
						Emulators[CurEmu].nGameTotals[CList][3]+=1;
				}
			}
		}
		fclose(fRMS);
	}
	myfree(pChecks);

	return	nRet;

}

int makermsfile(pGETFILE pGetFiles,int NoFound,char *pNeoText)
{
	FILE	*fRMS;
	FILE	*fNew;
	int	fsz;
	long	flen;
	static char	szRMS[64];
	int	bGotName;
	MAMEGAME	Game;
	pQUICKCHECK	pQuickChecks;

	int	NoChecks;
	int	MaxChecks;
	unsigned	long	lsize;
	int	n;
	size_t	noread;
	int	nRet=0;
	SOUND_UPDATE_INIT

//make the biggest buffer we can for our check file
	MaxChecks=MAX_GAMES;
	lsize=(unsigned long)((unsigned long)MaxChecks*(unsigned long)sizeof(QUICKCHECK));
	pQuickChecks=myalloc(lsize);


	if(!pQuickChecks)
	{
		printf("Failed to Allocate Memory for 'Quick Check' Game List\n");
		return nRet;
	}
	getrmsfile(szRMS);
	unlink("r$.tmp");

//see if we can fit the whole file in memory
	fsz=open(szRMS,O_RDONLY|O_BINARY);
	if(fsz)
	{
		flen=filelength(fsz);
		flen-=10;
		close(fsz);
	}

	fRMS=fopen(szRMS,"rb");
//buffer the existing file
	if(fRMS)
	{
		n=0;
//skip past format header
		fseek(fRMS,RMS_HEAD_SIZE,SEEK_SET);
		while(TRUE)
		{
			fgetpos(fRMS,&pQuickChecks[n].filepos);
			if(!fread(&Game,sizeof(MAMEGAME),1,fRMS))
				break;
			strcpy(pQuickChecks[n].GameName,Game.GameName);
			n++;
			if(n>=MaxChecks)
				break;
		}
		noread=n;
	} else
		noread=0;


	NoChecks=(int)noread;

	fNew=fopen("r$.tmp","wb");
//write header
	if(!fNew)
	{
		myfree(pQuickChecks);
		return nRet;
	}
	fwrite(RMSFORMAT,1,RMS_HEAD_SIZE,fNew);
	memset(Emulators[CurEmu].nGameTotals,0,sizeof(int)*NO_CUSTOM_LISTS*4);

	for(n=0;n<NoFound;n++)
	{
//get a nice display name
		bGotName=(int)pGetFiles[n].bGotName;
		if(bGotName||!noUnknowns||*Emulators[CurEmu].szPattern)
		{
			strcpy(Game.GameName,pGetFiles[n].FileName);
			loginfo(1,"game %d : %s = %s",n,Game.GameName,pGetFiles[n].szLongName);
			Game.Orientation=BLANK_ORIENT;
			addgame(&pGetFiles[n],&Game,fRMS,fNew,pQuickChecks,NoChecks,pNeoText);
			nRet++;
		} else
			loginfo(1,"name NOT found for %d : %s",n,pGetFiles[n].FileName);
		SOUND_UPDATE_TRIP(10)
	}

	myfree(pQuickChecks);

	fclose(fNew);
	fclose(fRMS);
	unlink(szRMS);
	rename("r$.tmp",szRMS);
	return nRet;
}

void setautocustom(pMAMEGAME pGame,int CustIdx)
{
	CustIdx-=CUSTOM_LIST_OFFSET;
	if(CustIdx<NO_CUSTOM_LISTS)
	{
		pGame->Orientation&=7;
		pGame->Orientation|=((CustIdx)<<4);
	}
}

int addgame(pGETFILE pGetFile,pMAMEGAME pGame,FILE *fRMS,FILE *fNew,pQUICKCHECK pQuickChecks,int NoChecks,char *pNeoText)
{
	fpos_t	fpos;
	int	nRet;
	int	CIndex,nTemp;

	nRet=FALSE;
	pGame->bRandomRun=FALSE;
//check for existing config
	nRet=rmsexist(pGetFile->FileName,pGame,fRMS,pQuickChecks,NoChecks);
	safecopy(pGame->szDispName,pGetFile->szLongName,63);

	safecopy(pGame->szParent,pGetFile->szParent,11);
	safecopy(pGame->szExtension,pGetFile->szExtension,4);
	pGame->nGameType=pGetFile->bType;
	pGame->bRomFolder=pGetFile->bRomFolder;
	fgetpos(fNew,&fpos);
	pGame->fFilePos=fpos;
	pGame->bChanged=FALSE;


	if(isneogeo(pGame->GameName,pNeoText))
		setautocustom(pGame,CustNeo);

	if(pGetFile->bVector)
		setautocustom(pGame,CustVector);


	if(!noAutoDisable&&!pGetFile->bOkay)
	{
		pGame->Orientation&=248;
		pGame->Orientation|=4;
	} else
	{
		nTemp=nAutoRotate-AUTO_ROTATE_OFFSET;
//don't rotate disabled games
		if(nTemp>0&&pGetFile->bRotated&&(pGame->Orientation&7)!=4)
		{
			pGame->Orientation&=248;
			pGame->Orientation|=nTemp;
		}
	}

	CIndex=(pGame->Orientation>>4)%NO_CUSTOM_LISTS;
	Emulators[CurEmu].nGameTotals[CIndex][0]+=1;

	switch(pGame->Orientation&7)
	{
		case	0:
			Emulators[CurEmu].nGameTotals[CIndex][1]+=1;
			break;
		case	1:
		case	2:
		case	3:
			Emulators[CurEmu].nGameTotals[CIndex][2]+=1;
			break;
		case	4:
			Emulators[CurEmu].nGameTotals[CIndex][3]+=1;
	}

	fwrite(pGame,sizeof(MAMEGAME),1,fNew);
	return	nRet;
}


char *getclones(char *szName,char *pGameText,char *szClone)
{
	char 	*pWork;
	char	*pRet;
	char	*pWork2;
	char	szWork[64];
	char	buffer[512];

	sprintf(buffer,"^%s{",szName);
	pRet=strstr(pGameText,buffer);

	if(pRet)
	{
		strncpy(szWork,pRet,32);
		szWork[32]=0;

		pWork=strstr(szWork,"{");
		pWork++;
		pRet=strstr(pRet,"{");
		pWork2=strstr(pWork,"{");
		if(pWork2)
			*pWork2=0;
		strcpy(szClone,pWork);


	}
	return pRet;
}

int rmsexist(char *szName,pMAMEGAME pGame,FILE *fin,pQUICKCHECK pQuickChecks,int NoChecks)
{
	int	n;

//set the stuff to default
	if(pGame->Orientation==BLANK_ORIENT)
		pGame->Orientation=0;
	pGame->bLocalConfig=FALSE;
	pGame->bRandomRun=FALSE;
	if(fin==NULL||pQuickChecks==NULL)
		return FALSE;

	n=0;
	while(TRUE)
	{
		if(*pQuickChecks[n].GameName==*szName && !strcmpi(szName,pQuickChecks[n].GameName))
		{
			fsetpos(fin,&pQuickChecks[n].filepos);
			fread(pGame,sizeof(MAMEGAME),1,fin);
			if(!ReRun)
				pGame->bRandomRun=FALSE;
			return TRUE;
		}
		n++;
		if(n>=NoChecks)
			break;
	}
	return FALSE;
}

void tablecom(char *szCom)
{
	emudir(NULL);
	system(szCom);
	menudir();
}

void getrealname(pGETFILE pGetFile,char *pGameList)
{
	char	*pWork;
	char	*pWork2;
	int	capital=FALSE;
	char	szCommand[512];

	pGetFile->bRotated=FALSE;
	pGetFile->bOkay=TRUE;
	pGetFile->bVector=FALSE;

	sprintf(szCommand,"^%s{",pGetFile->FileName);
	pWork=strstr(pGameList,szCommand);

	*pGetFile->szParent=0;
	if(pWork)
	{
		pGetFile->bGotName=TRUE;
		getitem(pGetFile->szLongName,pWork,1,63,'{');
		pWork2=strstr(pWork,"@");
		if(pWork2)
		{
			pWork2++;
			pGetFile->bRotated=(*pWork2&1);
			pGetFile->bOkay=(*pWork2&2);
			pGetFile->bVector=(*pWork2&4);
			getitem(pGetFile->szParent,pWork,2,11,'{');
		}

	} else
	{
		switch(Capitalise)
		{
			case CAPITAL_NONE:
				break;
			case CAPITAL_SHORT:
				if(!strcmpi(pGetFile->szLongName,pGetFile->FileName))
					capital=TRUE;
				break;
			case CAPITAL_LONG:
				if(strcmpi(pGetFile->szLongName,pGetFile->FileName))
					capital=TRUE;
				break;
			case CAPITAL_ALL:
				capital=TRUE;
		}
		if(capital)
			makenice(pGetFile->szLongName);
		pGetFile->bGotName=FALSE;
	}
}

void makenice(char *szIn)
{
	int n;
	int state=1;
	char szWork[2];
	szWork[1]=0;

	for(n=0;n<strlen(szIn);n++)
	{
		szWork[0]=szIn[n];
		if(state)
			strupr(szWork);
		else
			strlwr(szWork);
		state=isspace(*szWork);
		szIn[n]=szWork[0];
	}
}

void getres(char *szRes,int *pX,int *pY)
{
	char szWork[256];
	char *pWork;
	*pX=*pY=0;

	strcpy(szWork,szRes);
	pWork=strstr(szWork,"x");
	if(pWork)
	{
		*pWork=0;
		pWork++;
		*pX=atoi(szWork);
		*pY=atoi(pWork);
	}
}
