#ifndef _LCDLIB_H_
#define _LCDLIB_H_

//	LPT1	0x0378
//	LPT2	0x0278
//#define LPT1         0
//#define LPT2         1

#define LPORT			0x0278
#define LPT1			0x0100
#define LPT2			0x0000
#define LCD_DATA			LPORT+0
#define LCD_CONTROL		LPORT+2


#define CMD_CLR			0x01	// Clears all display and returns the cursor to the home position (Addr 0). 
#define CMD_HOME			0x02	// Returns the cursor to the home position (Address 0). 
									// Also returns the display being shifted to the original position.
									// DD RAM contents remain unchanged. 

#define CMD_ENTRY			0x04	// Sets the cursor move direction and specifies or not to shift the display.
									// These operations are performed during data read and write
#define CMD_ENTRY_INC		0x02	// Bit 1 ON  - Increment DD RAM Address when character code written
#define CMD_ENTTY_DEC		0x00	// Bit 1 OFF - Decrement "	"	"	"	"	"
#define CMD_ENTRY_SHFT		0x01	// Bit 0 ON  - Shift Display Right when INC, Left when DEC

#define CMD_ONOFF			0x08	// Sets ON/OFF display, cursor and cursor position character blink 
#define CMD_ONOFF_DSP_ON	0x04	// Bit 2 ON		- Set Display On
#define CMD_ONOFF_DSP_OFF	0x00	// Bit 2 OFF	- Set Display Off
#define CMD_ONOFF_CRS_ON	0x02	// Bit 1 ON		- Set Cursor On
#define CMD_ONOFF_CRS_OFF	0x00	// Bit 1 OFF	- Set Cursor Off
#define CMD_ONOFF_BNK_ON	0x01	// Bit 0 ON		- Set Blink On
#define CMD_ONOFF_BNK_OFF	0x00	// Bit 0 OFF	- Set Blink Off

#define CMD_SHFT			0x10	// Moves the cursor and shifts the display without changing DD RAM contents 
#define CMD_SHFT_DSP		0x08	// Bit 3 ON		- Shift Entire Display
#define CMD_SHFT_CUR		0x00	// Bit 3 OFF	- Shift Cursor only
#define CMD_SHFT_RIGHT		0x04	// Bit 2 ON		- Shift Display or Cursor Right
#define CMD_SHFT_LEFT		0x00	// Bit 2 OFF	- Shift Display or Cursor Left

#define CMD_FN			0x20	// Sets interface data length , number of display lines  and character font 
#define CMD_FN_8BIT		0x10	// Bit 4 ON		- Set to 8 bit interface
#define CMD_FN_4BIT		0x00	// Bit 4 OFF	- Set to 4 bit interface
#define CMD_FN_2LINE		0x08	// Bit 3 ON		- Set to 2 display lines
#define CMD_FN_1LINE		0x00	// Bit 3 OFF	- Set to 1 display line
#define CMD_FN_5X10		0x04	// Bit 2 ON		- Use 5x10 font (1 display line only)
#define CMD_FN_5X7		0x00	// Bit 2 OFF	- Use 5x7 font

#define CMD_CGR_ADR		0x40	// Sets the CG RAM address. CG RAM data is sent after this setting 
#define CMD_CGR_MASK		0x3F	// Mask for CG RAM address data - 0-63

#define CMD_DDR_ADR		0x80	// Sets the DD RAM address. DD RAM data is sent after this setting 
#define CMD_DDR_MASK		0x7F	// Mask for DD RAM address data - Line 1: 0-63, 2: 64-127


void ReadyPORT(int);
void ReadyLPT(int);
void WriteLPT (int, int, int, unsigned char);
void WriteChar(int,char);
void mssleep(int);
void WriteCOM(int,char*);
void displayLCDLogo(void);
void displayLCDGameInfo(char*);
void displayLCDShutdown(void);

#endif
