#include	"main.h"
#include "sbfm.h"

static BYTE regBaseTable[9]={0x0,0x1,0x2,0x8,0x9,0xa,0x10,0x11,0x12};

sbInst defInst;


static struct
{
	BYTE test;               // the test register, register 1
	BYTE timer1;             // timer1 register, register 2
	BYTE timer2;             // timer 2 register, register 3
	BYTE tmrctrl;            // timer control register, register 4
	BYTE spchsel;            // speech select/keyboard split, register 8
	BYTE fnumlow[9];       // freq. select low, registers 0xa0-0xa8
	BYTE fnumhikb[9];      // freq. hi/octave/keyon, registers 0xb0-0xb8
	BYTE rythmdep;           // rythm select/depth/drum keyon, register 0xbd
	BYTE feedconn[9];      // feedback/connection, registers 0xc0-0xc8
} FMRegs;                // only one of these!


void SetReg(BYTE theReg,BYTE theValue)
{
	int	n;
	outportb(0x388,theReg);
	for(n=0;n<7;n++)
		inportb(0x388);
	outportb(0x389,theValue);
	for(n=0;n<35;n++)
		inportb(0x389);
}


int DetectFM()
{
	BYTE stat1;
	BYTE stat2;
	BYTE result=0;

	SetReg(0x1,0);                     // initialize test register
	SetReg(0x4,0x60);                  // reset both timers
	SetReg(0x4,0x80);                  // enable timer interrupts
	stat1=inportb(0x388);             // read the status port
	SetReg(0x2,0xff);                  // write ff to the timer 1 count
	SetReg(0x4,0x21);                  // start timer 1
	delay(900);
	stat2=inportb(0x388);             // read the status port
	SetReg(0x4,0x60);                  // reset both timers
	SetReg(0x4,0x80);                  // enable timer interrupts
	stat1&=0xe0;
	if (!stat1)
	{
		stat2&=0xe0;
		if(stat2==0xc0)
		{
			result++;
			memset(&defInst,0,sizeof(defInst));
			strcpy(defInst.fileId,"SBI");
			strcpy(defInst.instName,"SBFM Default Voice");
			defInst.modParams=0x01;
			defInst.carParams=0x11;
			defInst.modScalOut=0x4F;
			defInst.carScalOut=0x0;
			defInst.modAttkDec=0xF1;
			defInst.carAttkDec=0xF2;
			defInst.modSustRel=0x53;
			defInst.carSustRel=0x74;
			defInst.modWaveSel=0x0;
			defInst.carWaveSel=0x0;
			defInst.feedBkConn=0x0;
		}
	}
	return result;
}


void ResetFM()
{
	BYTE chan;

	memset(&FMRegs,0,sizeof(FMRegs));
	FMRegs.test=0;
	SetReg(1,FMRegs.test);
	FMRegs.rythmdep=0xc0;
	SetReg(0xbd,FMRegs.rythmdep);
	FMRegs.spchsel=0x0;
	SetReg(0x8,FMRegs.spchsel);
	for(chan=0;chan<9;chan++)
	{
		SetReg(0xa0+chan,FMRegs.fnumlow[chan]);
		SetReg(0xb0+chan,FMRegs.fnumhikb[chan]);
		SetReg(0xc0+chan,FMRegs.feedconn[chan]);
	}
	for(chan=1;chan<=9;chan++)
		ProgramChange(chan,&defInst);
	return;
}


void SetMusicMode(void)
{
	FMRegs.spchsel&=0x7f;
	FMRegs.rythmdep&=0xdf;
	SetReg(0x8,FMRegs.spchsel);
	SetReg(0xbd,FMRegs.rythmdep);
}



void ProgramChange(BYTE theChannel,sbInst *theInstTable)
{
	BYTE targetReg;
	BYTE nindex;

	nindex=theChannel-1;

	SetReg(1,0x20);
	targetReg=regBaseTable[nindex]+0x20;
	SetReg(targetReg,theInstTable->modParams);
	targetReg+= 0x3;
	SetReg(targetReg,theInstTable->carParams);
	targetReg+= 0x1d;
	SetReg(targetReg,theInstTable->modScalOut);
	targetReg+= 0x3;
	SetReg(targetReg,theInstTable->carScalOut);
	targetReg+= 0x1d;
	SetReg(targetReg,theInstTable->modAttkDec);
	targetReg+= 0x3;
	SetReg(targetReg,theInstTable->carAttkDec);
	targetReg+= 0x1d;
	SetReg(targetReg,theInstTable->modSustRel);
	targetReg+= 0x3;
	SetReg(targetReg,theInstTable->carSustRel);
	targetReg+= 0x5d;
	SetReg(targetReg,theInstTable->modWaveSel);
	targetReg+= 0x3;
	SetReg(targetReg,theInstTable->carWaveSel);

	return;
}


void SetFreq(BYTE theChannel,int theFNum)
{
	BYTE nindex=theChannel-1;

	FMRegs.fnumlow[nindex]=(BYTE)theFNum;
	theFNum>>=8;
	theFNum&=3;
	FMRegs.fnumhikb[nindex]|=(BYTE)theFNum;
	SetReg((0xa0+nindex),FMRegs.fnumlow[nindex]);
	SetReg((0xb0+nindex),FMRegs.fnumhikb[nindex]);
	return;
}

void SetOct(BYTE theChannel,BYTE theOctave)
{
	BYTE nindex=theChannel-1;

	if((theOctave<8)&&((char)theOctave>-1))
	{
		theOctave<<=2;
		FMRegs.fnumhikb[nindex]&=0xe3;
		FMRegs.fnumhikb[nindex]|=theOctave;
		SetReg((0xb0+nindex),FMRegs.fnumhikb[nindex]);
	}
	return;
}

void KeyOn(BYTE theChannel)
{
	BYTE nindex=theChannel-1;

	FMRegs.fnumhikb[nindex]|=0x20;
	SetReg((0xb0+nindex),FMRegs.fnumhikb[nindex]);
	return;
}

void KeyOff(BYTE theChannel)
{
	BYTE nindex=theChannel-1;

	FMRegs.fnumhikb[nindex]&=0xdf;
	SetReg((0xb0+nindex),0);
	return;
}
