#include "main.h"
#include "screen.h"
#include "disp.h"
#include "pal.h"

extern BYTE DimPalette[256];
static unsigned int winpagesize=0;

static MODEINFO modeinfo;

int getmodeinfo(int mode)
{
	int	n;
	unsigned long	seg;
	int	sel;


	__dpmi_regs _dpmi_reg;

	seg=__dpmi_allocate_dos_memory(16384/16,&sel);

	memset(&modeinfo,0,sizeof(MODEINFO));
	dosmemput(&modeinfo,sizeof(MODEINFO),((seg<<4)&0xfffff));


	_dpmi_reg.x.ax=0x4F01;
	_dpmi_reg.x.di=0;
	_dpmi_reg.x.es=seg;
	_dpmi_reg.x.cx=mode;
	__dpmi_int(0x10,&_dpmi_reg);

	if(_dpmi_reg.h.ah)
	{
		__dpmi_free_dos_memory(sel);
		winpagesize=16384;
		return FALSE;
	}

	dosmemget(((seg<<4)&0xfffff), sizeof(MODEINFO), &modeinfo);
	winpagesize=(modeinfo.WinSize*1024)/4;
	__dpmi_free_dos_memory(sel);

	if(!winpagesize)
		winpagesize=16384;
	return TRUE;
}

void centermode(char adjustx,char adjusty)
{
	if(ReRun)
		return;
	switch(BIOSCab)
	{
		case	ARCADE_MODE:
			loginfo(1,"Setting Arcade Mode timings");
			callAOSBios(SET_VH_TOTALS,ArcadeH,ArcadeV);
			break;
		case	PAL_MODE:
			loginfo(1,"Setting PAL TV Mode timings");
			callAOSBios(SET_VH_TOTALS,PalH,PalV);
			break;
	}

//pass the info on to our BIOS
	callAOSBios(SET_VH_SCANLINE,ScanH,ScanV);
	callAOSBios(SET_CENTER,adjustx,adjusty);
}


void getpal(pBYTE table,int colors)
{
	memcpy(table,intpal,colors*3);
}

void setpcxpal(pBYTE table)
{
	int	n;
	for(n=0;n<256;n++)
	{
		outportb(0x3c8,n);
		outportb(0x3c9,(*table++)&63);
		outportb(0x3c9,(*table++)&63);
		outportb(0x3c9,(*table++)&63);
	}
}

void putsvga(void)
{
	int	n;
	unsigned long *pData, saddress,lvalue;
	int pos,x,y;
	int work;

	pBYTE pDataByte;

//check if we're doing any flipping
	if(!noflipx)
	{
		if(!noflipy)
		{
			putvesaflipxy();
			return;
		}
		putvesaflipx();
		return;
	} else if(!noflipy)
	{
			putvesaflipy();
			return;
	}
//normal blit
	saddress=SCREEN_START;
	pData=(unsigned long *)screen;

	if(!noScanLines)
	{
		n=pos=0;
		setsvgapage(n);

		_farsetsel(_dos_ds);
		for(y=0;y<480;y++)
		{
			if(y&1)
			{
				pDataByte=(pBYTE)pData;
				for(x=0;x<X_QUAD;x++)
				{
					unsigned long val1,val2,val3,val4;
					val1=DimPalette[*(pDataByte++)];
					val2=DimPalette[*(pDataByte++)];
					val3=DimPalette[*(pDataByte++)];
					val4=DimPalette[*(pDataByte++)];
					lvalue=val1|val2<<8|val3<<16|val4<<24;
					if(pos>(winpagesize-1))
					{
						pos=0;
						saddress=SCREEN_START;
						setsvgapage(++n);
					}
					_farnspokel(saddress,lvalue);
					saddress+=4;
					pos++;
				}
				pData+=X_QUAD;
			} else if((pos+X_QUAD)<winpagesize)
			{
				_dosmemputl(pData,X_QUAD,saddress);
				pos+=X_QUAD;
				saddress+=X_BYTE;
				pData+=X_QUAD;
			} else
			{
//split across page
				work=winpagesize-pos;
//firstpart
				_dosmemputl(pData,work,saddress);
//page flip
				setsvgapage(++n);
//secomd part
				saddress=SCREEN_START;
				pos=0;
				pData+=work;
				_dosmemputl(pData,X_QUAD-work,saddress);
				pos+=(X_QUAD-work);
				saddress+=(X_QUAD-work)<<2;
				pData+=(X_QUAD-work);
			}
		}
	} else
	{
		for(n=0;n<(65536/winpagesize);n++)
		{
			setsvgapage(n);
			_dosmemputl(pData,winpagesize,saddress);
			pData+=winpagesize;
		}
		setsvgapage(n);
		_dosmemputl(pData,11264,saddress);
	}

}

void putvesaflipxy()
{
	int	n;
	int x,y,outpos;
	unsigned long *pData;
	unsigned char *pWork;
	unsigned long saddress,lout;

	saddress=SCREEN_START;
	pData=(unsigned long *)screen;

	n=0;
	pData+=((480*X_QUAD)-1);
	setsvgapage(n);
	for(y=0;y<480;y++)
	{
		if(saddress>SCREEN_CHECK)
		{
			ASSIGN_PWORK
			if(SCANLINE)
			{
				BLIT_X_LINE
				DIM_QUAD
				BLIT_FLIPXY
			} else
			{
				BLIT_X_LINE
				NORMAL_QUAD
				BLIT_FLIPXY
			}
		} else
		{
			ASSIGN_PWORK
			if(SCANLINE)
			{
				BLIT_X_LINE
				DIM_QUAD
				BLIT_FLIPXY_NOCHECK
			} else
			{
				BLIT_X_LINE
				NORMAL_QUAD
				BLIT_FLIPXY_NOCHECK
			}
		}
	}
}

void putvesaflipx()
{
	int	n;
	int x,y,outpos;
	unsigned long *pData;
	unsigned char *pWork;
	unsigned long saddress,lout;
	saddress=SCREEN_START;
	pData=(unsigned long *)screen;

	n=0;
	setsvgapage(n);
	for(y=0;y<480;y++)
	{
		pData+=159;
		if(saddress>SCREEN_CHECK)
		{
			ASSIGN_PWORK
			if(SCANLINE)
			{
				BLIT_X_LINE
				DIM_QUAD
				BLIT_FLIPX
			}
			else
			{
				BLIT_X_LINE
				NORMAL_QUAD
				BLIT_FLIPX
			}
		}
		else
		{
			ASSIGN_PWORK
			if(SCANLINE)
			{
				BLIT_X_LINE
				DIM_QUAD
				BLIT_FLIPX_NOCHECK
			}
			else
			{
				BLIT_X_LINE
				NORMAL_QUAD
				BLIT_FLIPX_NOCHECK
			}
		}
		pData+=161;
	}
}

void putvesaflipy()
{
	int	n;
	int x,y,outpos;
	unsigned long *pData;
	pBYTE pWork;
	unsigned long saddress,lout;
	saddress=SCREEN_START;
	pData=(unsigned long *)screen;

	pData+=(479*X_QUAD);
	n=0;
	setsvgapage(n);
	for(y=0;y<480;y++)
	{
		if(saddress>SCREEN_CHECK)
		{
			ASSIGN_PWORK
			if(SCANLINE)
			{
				for(x=0;x<X_QUAD;x++)
				{
					lout=(DimPalette[pWork[0]]<<24)|(DimPalette[pWork[1]]<<16)|(DimPalette[pWork[2]]<<8)|(DimPalette[pWork[3]]);
					_farpokel(_dos_ds, saddress, lout);
					saddress+=4;
					if(saddress>=SCREEN_END)
					{
						n++;
						setsvgapage(n);
						saddress=SCREEN_START;
					}
					pData++;
					pWork+=4;
				}
			} else
			{
				for(x=0;x<X_QUAD;x++)
				{
					_dosmemputl(pData,1,saddress);
					saddress+=4;
					if(saddress>=SCREEN_END)
					{
						n++;
						setsvgapage(n);
						saddress=SCREEN_START;
					}
					pData++;
				}
			}
  			pData-=320;
		} else
		{
			ASSIGN_PWORK
			if(SCANLINE)
			{
				BLIT_X_LINE
				DIM_YFLIP_QUAD
				BLIT_FLIPY
			} else
			{
				_dosmemputl(pData,X_QUAD,saddress);
				saddress+=X_BYTE;
			}
			pData-=X_QUAD;
		}
	}
}


void put(pBYTE pData,short seg)
{
	int dummy1,dummy2;

//check if we're doing any flipping
	if(!noflipx)
	{
		if(!noflipy)
		{
			putflipxy(pData,seg);
			return;
		}
		putflipx(pData,seg);
		return;
	} else if(!noflipy)
	{
		putflipy(pData,seg);
		return;
	}
//got to here - normal blit
	__asm__ __volatile__
	(
		".align 4 \n"
//save es and set our selector
		"pushw		%%es	\n"
		"movw		%%ax,%%es \n"

		"movw		$0x3da,%%dx \n"
//wait for retace
		"movw		$0x3da,%%dx \n"
"2:\n"
		"inb		%%dx,%%al \n"
		"testb		$8,%%al \n"
		"jz			2b \n"

//setup screen pointer
		"movl		$0xa0000,%%edi \n"
//copy screen across
		"movl		$16000,%%ecx \n"
		"rep \n"
		"movsl \n"
//restore es
		"popw		%%es \n"

//outputs (clobbered inputs)
		: "=a" (dummy1),
		  "=S" (dummy2)
// inputs -
//ax = selector
//esi = src
		:"0" (seg),
		 "1" (pData)
// registers modified
		:  "cx", "dx", "di", "cc", "memory"
	);
}

void putflipxy(pBYTE pData,short seg)
{
	int	dummy1,dummy2;

	__asm__ __volatile__
	(
		".align 4 \n"
//save es and set our selector
		"pushw	%%es	\n"
		"movw		%%ax,%%es \n"

		"movw		$0x3da,%%dx \n"
//wait for retace
		"movw		$0x3da,%%dx \n"
"2:\n"
		"inb		%%dx,%%al \n"
		"testb	$8,%%al \n"
		"jz			2b \n"

//setup screen pointer
		"movl		$0xa0000,%%edi \n"
//copy screen across
		"addl		$63999,%%esi \n"
		"movl		$64000,%%ecx \n"
"3:\n"
		"movb		%%ds:(%%esi),%%al \n"
		"movb		%%al,%%es:(%%edi) \n"
		"movb		%%ds:(%%esi),%%al \n"
		"incl		%%edi \n"
		"decl		%%esi \n"
		"loop		3b\n"
//restore es
		"popw		%%es \n"
//outputs  (clobbered inputs)
		: "=a" (dummy1),
		  "=S" (dummy2)
// inputs -
//ax = selector
//esi = src
		:"0" (seg),
		 "1" (pData)
// registers modified
		: "cx", "dx", "di", "cc", "memory"
	);
}

void putflipy(pBYTE pData,short seg)
{
	int dummy1,dummy2;
	__asm__ __volatile__
	(
		".align 4 \n"
//save es and set our selector
		"pushw		%%es \n"
		"movw		%%ax,%%es \n"

		"movw		$0x3da,%%dx \n"
//wait for retace
		"movw		$0x3da,%%dx \n"
"2:\n"
		"inb		%%dx,%%al \n"
		"testb		$8,%%al \n"
		"jz			2b \n"

//setup screen pointer
		"movl		$0xa0000,%%edi \n"
//copy screen across
		"movl		$200,%%ecx \n"
		"addl		$64320,%%esi \n"
"3:\n"
		"subl		$640,%%esi \n"
		"movl		$320,%%edx \n"
"4:\n"
		"movb		%%ds:(%%esi),%%al \n"
		"movb		%%al,%%es:(%%edi) \n"
		"movb		%%ds:(%%esi),%%al \n"
		"incl		%%edi \n"
		"incl		%%esi \n"
		"decw		%%dx \n"
		"cmpw		$0,%%dx \n"
		"jne		4b\n"
		"loop		3b\n"
//restore es
		"popw		%%es \n"

//outputs  (clobbered inputs)
		: "=a" (dummy1),
		  "=S" (dummy2)
// inputs -
//ax = selector
//esi = src
		:"0" (seg),
		 "1" (pData)
// registers modified
		: "cx", "di", "cc", "memory"
	);

}

void putflipx(pBYTE pData,short seg)
{
	int dummy1,dummy2;
	__asm__ __volatile__
	(
		".align 4 \n"
//save es and set our selector
		"pushw	%%es	\n"
		"movw		%%ax,%%es \n"

		"movw		$0x3da,%%dx \n"
//wait for retace
		"movw		$0x3da,%%dx \n"
"2:\n"
		"inb		%%dx,%%al \n"
		"testb	$8,%%al \n"
		"jz			2b \n"

//setup screen pointer
		"movl		$0xa0000,%%edi \n"
//copy screen across
		"movl		$200,%%ecx \n"
		"subl		$320,%%esi \n"
"3:\n"
		"addl		$640,%%esi \n"
		"movl		$320,%%edx \n"
"4:\n"
		"movb		%%ds:(%%esi),%%al \n"
		"movb		%%al,%%es:(%%edi) \n"
		"movb		%%ds:(%%esi),%%al \n"
		"incl		%%edi \n"
		"decl		%%esi \n"
		"decw		%%dx \n"
		"cmpw		$0,%%dx \n"
		"jne		4b\n"
		"loop		3b\n"
//restore es
		"popw		%%es \n"

//outputs  (clobbered inputs)
		: "=a" (dummy1),
		  "=S" (dummy2)
// inputs -
//ax = selector
//esi = src
		:"0" (seg),
		 "1" (pData)
// registers modified
		: "cx", "dx", "di", "cc", "memory"
	);
}


void blit(pBYTE source,int x,int y,pBYTE dest,BYTE color,int w,int h,int dbly)
{
	int	xlp,ylp,nindex=0;
	pBYTE pWork;
	int	move=ScreenOrient.xpixel-w;

	if(x<0||x>=(ScreenOrient.xpixel-w)||y<0||y>=(ScreenOrient.ypixel-h))
		return;

	pWork=dest+x+(y*ScreenOrient.xpixel);
	for(ylp=0;ylp<h;ylp++)
	{
		for(xlp=0;xlp<w;xlp++,nindex++,pWork++)
		{
			if(source[nindex])
				*pWork=color;
		}
		if(dbly)
		{
			nindex-=xlp;
			pWork+=move;
			for(xlp=0;xlp<w;xlp++,nindex++,pWork++)
			{
				if(source[nindex])
					*pWork=color;
			}
		}
		pWork+=move;
	}
}

void tblit(pBYTE source,int x,int y,pBYTE dest,BYTE transcolor,int w,int h)
{
	int	xlp,ylp,nindex=0;
	int	move=ScreenOrient.xpixel-w;
	pBYTE pWork;


	if(x<0||x>=(ScreenOrient.xpixel-w)||y<0||y>=(ScreenOrient.ypixel-h))
		return;

	pWork=dest+x+(y*ScreenOrient.xpixel);


	for(ylp=0;ylp<h;ylp++)
	{
		for(xlp=0;xlp<w;xlp++,nindex++,pWork++)
		{
			if(source[nindex]!=transcolor)
				*pWork=source[nindex];
		}
		pWork+=move;
	}
}

void fastback(void)
{
	fastbackact(pcxback.image,screen,DataSeg,Hires?76800:16000);
}

void fastbackact(pBYTE source,pBYTE dest,short seg,int ssize)
{
	int	dummy1,dummy2,dummy3,dummy4;

	__asm__ __volatile__
	(
//save es and set our selector
		"pushw	%%es	\n"
		"movw	%%ax,%%es \n"

		"rep \n"
		"movsl \n"

		"popw	%%es \n"

/* outputs  (clobbered inputs)*/
		: "=a" (dummy1),
		  "=c" (dummy2),
		  "=S" (dummy3),
		  "=D" (dummy4)

/* inputs -
 ax = selector
 esi = src, edi = dest*/
		:"0" (seg),
		 "1" (ssize),
		 "2" (source),
		 "3" (dest)
/* registers modified */
		: "cc", "memory"
	);

}


void movecenter(void)
{
	int left,right,up,down;
//make the movements logical for the given screen orientation
	switch(ScreenOrient.Orientation)
	{
		case	1:
			left=keys[K_UP]|keys[K_UP2];
			right=keys[K_DOWN]|keys[K_DOWN2];
			up=keys[K_RIGHT]|keys[K_RIGHT2];
			down=keys[K_LEFT]|keys[K_LEFT2];
			break;
		case	2:
			left=keys[K_DOWN]|keys[K_DOWN2];
			right=keys[K_UP]|keys[K_UP2];
			up=keys[K_LEFT]|keys[K_LEFT2];
			down=keys[K_RIGHT]|keys[K_RIGHT2];
			break;
		default:
			left=keys[K_LEFT]|keys[K_LEFT2];
			right=keys[K_RIGHT]|keys[K_RIGHT2];
			up=keys[K_UP]|keys[K_UP2];
			down=keys[K_DOWN]|keys[K_DOWN2];
	}

	if(left&&nCenterX<8)
		movescreen(1,0);
	if(right&&nCenterX>-8)
		movescreen(-1,0);
	if(up&&nCenterY<16)
		movescreen(0,1);
	if(down&&nCenterY>-16)
		movescreen(0,-1);

}

void movescreen(int xmove,int ymove)
{
	BYTE curhrs;
	int	outval;
	int	vrt_start,vrt_end,temp,vrt,temp2;

	outportb(0x3d4,4);
	curhrs=inportb(0x3d5);
	curhrs+=xmove;

	clrcrtcprot();
//horizontal
	outval=4|(curhrs<<8);
  	outportw(0x3d4,outval);
	curhrs+=10;
	outval=5|((curhrs&31)<<8);
	outportw(0x3d4,outval);
//vertcal
/* get the vt retrace */
	outportb(0x3d4,0x10);

	vrt_start = inportb(0x3d5);
	vrt_start += ymove;

	outportb(0x3d4,0x11);
	vrt_end = inportb(0x3d5);
	temp = vrt_end & 0x0f;
	temp += ymove;
	vrt_end &= ~0x0f;
	vrt_end |= (temp & 0x0f);

	outportb(0x3d4,0x10);
	outportb(0x3d5,(vrt_start &0xff));
	if(!Hires)
	{
		outportb(0x3d4,0x11);
		outportb(0x3d5,vrt_end);
	}

	nCenterX += xmove;
	nCenterY += ymove;
	centermode((char)xmove,(char)ymove);
}

void settiming(void)
{
	 clrcrtcprot();
	 outportb(0x3d4,0x00);
	 outportb(0x3d5,ArcadeH);
	 outportb(0x3d4,0x06);
	 outportb(0x3d5,ArcadeV);
}
void setsvgapage(int page)
{
	__dpmi_regs r;
	r.x.ax=0x4f05;
	r.x.bx=0;
	r.x.dx=page;
	__dpmi_int(0x10, &r);
}

void clrcrtcprot(void)
{
	int	val;
	outportb(0x3d4,0x11);
	val=(inportb(0x3d5)&127);
	val<<=8;
	val|=0x11;
	outportw(0x3d4,val);
}
