;************************************************************************
;*                                                                      *
;* Microsoft Sidewinder Gamepad 32 bit driver				*
;* Version 1.1								*
;*                                                                      *
;* The information used in creating this driver was obtained from	*
;* William Grubb's information - http://www.vt.edu:10021/R/rogrubb3/	*
;* sidewind.htm								*
;*									*
;* This code comes without any warranty whatsoever. Use at your own	*
;* risk. That means it if blows up the entire world, it's your fault,	*
;* not mine.								*
;*									*
;* This code is *PUBLIC DOMAIN*. It has no restrictions. You can modify	*
;* it and redistribute it in source or object form.			*
;*                                                                      *
;************************************************************************
;*                                                                      *
;* Release history:                                                     *
;*                                                                      *
;* 05/13/97 - Release 1.0						*
;*									*
;* Initial release							*
;*									*
;************************************************************************
;*									*
;* This emulator is designed to assemble with the Netwide Assembler 	*
;* NASM	0.97								*
;*									*
;* For stack calling conventions, use:					*
;*									*
;* nasm -f (output mode) -dUSE_STACK sidewind.asm			*
;*									*
;* For register calling conventions (like Watcom), use:			*
;*									*
;* nasm -f (output mode) sidewind.asm					*									*
;*									*
;************************************************************************

; For assembly by NASM only

%define	DUMP_SIZE	200
%define	JOYSTICK_IO	0201h

bits 32
		section	.data

swDump		times (DUMP_SIZE) db 0 	; Nulls for our sampling
swData		times (DUMP_SIZE) db 0	; Sidewinder data pointer
returnCode	dd	0		; Return code

		section	.text

;************************************************************************
;*
;* Name : ParityCheckAndStore
;*
;* Entry: EDX=Button status, EDI=Button status ptr
;*
;* Exit : EDI Is advanced, and button status may or may not be stored
;*
;* Description:
;*
;* This routine checks to see if the parity of the data coming back from
;* the sidewinder is stored properly. If it isn't, the button status is not
;* updated. Otherwise, it is!
;*
;************************************************************************/

ParityCheckAndStore:
		mov	ebx, edx	; Get button status from EDX
		mov	ecx, ebx	; Dupe button status
		and	ch, 7fh		; Knock out parity bit from data
		xor	cl, ch
		setp	cl
		bt	ebx, 15
		setc	ch
		cmp	cl, ch
		jne	noStore

		mov	[ds:edi], edx	; Store our button status

noStore:
		add	edi, 4
		ret

;************************************************************************
;*
;* Name : GetSidewinderStatus
;*
;* Entry: EAX=Address of sidewinder structure (or on stack if USE_STACK)
;*	  is defined
;*
;* Exit : EAX=# Of Sidewinders found, otherwise 0fffffffffh if no Sidewinders
;*	  attached
;*
;* Description:
;*
;* This routine will get the attached Sidewinder's status (if it's attached)
;* and fill in the status structure pointed to by the value on input.
;*
;************************************************************************/

		global	GetSidewinderStatus_
		global	_GetSidewinderStatus


GetSidewinderStatus_:
_GetSidewinderStatus:

%ifdef USE_STACK
		mov	eax, [esp+4]
%endif

		push	ebx		; Save important registers
		push	ecx
		push	edx
		push	esi
		push	edi

		mov	ebx, eax	; Store the address of our structure
		mov	edi, swDump	; Offset to dump our rotary encoder data
		mov	ecx, DUMP_SIZE	; Size of dump
		mov	dx, JOYSTICK_IO	; Joystick I/O port
		cld

		pushf
		cli			; Stop ints so we get accurate readings

sampleLoop:
		out	dx, al		; Read strobe (trigger)
		in	al, dx		; Get our data
		mov	[ds:edi], al	; Store our received data
		inc	edi		; Increment our pointer
		dec	ecx		; Decrement our count
		jnz	sampleLoop	; Keep going until we're done

		popf			; Restore interrupt status

; Now we look for 15 high cycles in the stream we've just read in

		mov	esi, swDump	; Point to our dump area
		mov	ecx, DUMP_SIZE	; # Of bytes we've tried
		xor	edx, edx	; Zero our high cycle count

cycleLoop:
		mov	al, [esi]	; Get our received data
		inc	esi		; Increment our source pointer
		dec	ecx		; Decrement our count
		jz	noSidewinder	; No sidewinder attached!

		test	al, 10h		; Look for zero (active low)
		jnz	strobeHigh	; Strobe is high!

		xor	edx, edx	; Zero our count
		jmp	short cycleLoop	; Continue until we find it

strobeHigh:
		inc	edx		; Increment our high cycle count
		cmp	edx, 15		; 15 highs yet?
		jne	cycleLoop	; Nope! Keep going until we find it

; We now have our 15 cycles in a row. Now look for the leading edge of the
; data strobe.

		xor	edi, edi	; Zero our offset

dataStrobeLoop:
		mov	al, [esi]	; Get our data
		inc	esi		; Next ptr, please

		dec	ecx		; Decrement our remaining byte count
		jz	noSidewinder	; If we're out of data, no sidewinder

		test	al, 10h		; Anything there?
		jnz	dataStrobeLoop	; Nope!

		xor	edx, edx	; Zero our count

; We have the beginning of our data strobe! Let's start looking for a string
; of 15 zeros

highStrobeLoop:
		inc	edx		; Increment our count
		cmp	edx, 15		; 15 cycles?
		je	modeCheck	; Go do a Sidewinder mode check

		mov	al, [esi]	; Get our data
		inc	esi		; Increment our pointer

		dec	ecx		; Decrement our remaining byte count
		jz	noSidewinder	; If we're out of data, no sidewinder

		test	al, 10h		; Anything there?
		jz	highStrobeLoop	; Nope....

		mov	[ds:edi+swData], al	; Store our data
		inc	edi		; Increment our pointer
		jmp	short dataStrobeLoop

noSidewinder:
		mov	[ds:returnCode], dword 0ffffffffh ; Not found ;-(

swReturn:
		mov	eax, [ds:returnCode]
		pop	edi
		pop	esi
		pop	edx
		pop	ecx
		pop	ebx

		ret

; Now we know we've got valid data. Let's go through and see what we've got
; in terms of controllers and update any of the appropriate controller info

modeCheck:
		mov	esi, swData	; Our Sidewinder data
		mov	ecx, edi	; Store this here!
		mov	edi, ebx	; Our target storage address

		cmp	ecx, 60		; Mode A with 4 sidewinders?
		je	ModeA4
		cmp	ecx, 45		; Mode A with 3 sidewinders?
		je	ModeA3
		cmp	ecx, 30		; Mode A with 2 sidewinders?
		je	near ModeA2
		cmp	ecx, 15		; Mode A with 1 sidewinder?
		je	near ModeA1

; Mode B stuff

		cmp	ecx, 20		; Mode B with 3 sidewinders?
		je	near ModeB3
		cmp	ecx, 10		; Mode B with 2 sidewinders?
		je	near ModeB2
		cmp	ecx, 5		; Mode B with 1 sidewinder?
		je	near ModeB1

		jmp	short noSidewinder	; Not recognized - not installed

; 4 Sidewinders - Mode A

ModeA4:
		call	ModeAConvert	; Go do a Mode A conversion
		call	ParityCheckAndStore
		call	ModeAConvert	; Go do a Mode A conversion
		call	ParityCheckAndStore
		call	ModeAConvert	; Go do a Mode A conversion
		call	ParityCheckAndStore
		call	ModeAConvert	; Go do a Mode A conversion
		call	ParityCheckAndStore
		mov	[ds:returnCode], dword 4
		jmp	swReturn

; 3 Sidewinders - Mode A

ModeA3:
		call	ModeAConvert	; Go do a Mode A conversion
		call	ParityCheckAndStore
		call	ModeAConvert	; Go do a Mode A conversion
		call	ParityCheckAndStore
		call	ModeAConvert	; Go do a Mode A conversion
		call	ParityCheckAndStore
		mov	[ds:edi], dword 0
		mov	[ds:returnCode], dword 3
		jmp	swReturn

; 2 Sidewinders - Mode A

ModeA2:
		call	ModeAConvert	; Go do a Mode A conversion
		call	ParityCheckAndStore
		call	ModeAConvert	; Go do a Mode A conversion
		call	ParityCheckAndStore
		mov	[ds:edi], dword 0
		mov	[ds:edi+4], dword 0
		mov	[ds:returnCode], dword 2
		jmp	swReturn

; 1 Sidewinder - Mode A

ModeA1:
		call	ModeAConvert	; Go do a Mode A conversion
		call	ParityCheckAndStore
		mov	[ds:edi], dword 0
		mov	[ds:edi+4], dword 0
		mov	[ds:edi+8], dword 0
		mov	[ds:returnCode], dword 1
		jmp	swReturn


; 3 Sidewinders - Mode B

ModeB3:
		call	ModeBConvert	; Go do a Mode B conversion
		call	ParityCheckAndStore
		call	ModeBConvert	; Go do a Mode B conversion
		call	ParityCheckAndStore
		call	ModeBConvert	; Go do a Mode B conversion
		call	ParityCheckAndStore
		mov	[ds:edi], dword 0
		mov	[ds:returnCode], dword 3
		jmp	swReturn

; 2 Sidewinders - Mode B

ModeB2:
		call	ModeBConvert	; Go do a Mode B conversion
		call	ParityCheckAndStore
		call	ModeBConvert	; Go do a Mode B conversion
		call	ParityCheckAndStore
		mov	[ds:edi], dword 0
		mov	[ds:edi+4], dword 0
		mov	[ds:returnCode], dword 2
		jmp	swReturn

; 1 Sidewinder - Mode B

ModeB1:
		call	ModeBConvert	; Go do a Mode B conversion
		call	ParityCheckAndStore
		mov	[ds:edi], dword 0
		mov	[ds:edi+4], dword 0
		mov	[ds:edi+8], dword 0
		mov	[ds:returnCode], dword 1
		jmp	swReturn

		end

;************************************************************************
;*
;* Name : ModeAConvert
;*
;* Entry: ESI=Source data ptr
;*
;* Exit : EDX=Button status
;*
;* Description:
;*
;* This routine does the Mode A type conversion
;*
;************************************************************************/

ModeAConvert:
		xor	edx, edx	; Zero our mask
		xor	eax, eax	; Zero this for the OR
		mov	ecx, 15		; 15 Bits!

bitLoop:
		mov	al, [esi]	; Get our source bye
		inc	esi
		rcr	al, 6
		cmc
		rcr	dx, 1
		dec	ecx
		jnz	bitLoop

		ret

;************************************************************************
;*
;* Name : ModeBConvert
;*
;* Entry: ESI=Source data
;*
;* Exit : EDX=Button data
;*
;* Description:
;*
;* This routine does the Mode B type conversion
;*
;************************************************************************/


ModeBConvert:
		xor	edx, edx	; Zero our mask
		mov	ch, 05		; 5 Loops of 3 loops

startLoop:
		mov	al, 20h		; Start with this bitmask
		mov	cl, 03h		; 3 Loops!

innerLoop:
		test	[esi], al	; Is it on?
		clc
		jnz	notOn
		stc
notOn:
		rcr	dx, 1		; Suck up our new bit

		dec	cl
		shl	al, 1		; Move our bitmask over
		jnz	innerLoop	; Keep looping!

		inc	esi		; Next byte, please
		dec	ch		; Decrement our outer loop
		jnz	startLoop	; Try again

		ret
