#include "main.h"
#include "sbfm.h"

LPAUDIOWAVE lpWave[NO_SAMPLES];
HAC hVoice[NO_CHANNELS];
char szSamples[6][16];
int sampvol[6];
int sampfreq[6];
int vol;
extern int SBPresent;

void updatesound(void)
{
   AUpdateAudio();
}

int initsoundinterface(void)
{
   AUDIOINFO info;
   int n;
   static char szWork[512];


   AInitialize();

   info.nDeviceId=soundcard;
   info.wFormat=AUDIO_FORMAT_8BITS | AUDIO_FORMAT_MONO;
   info.nSampleRate=samplerate;

   if(AOpenAudio(&info)!=AUDIO_ERROR_NONE)
   {
      printf("Unable to initialise sound device\n(soundcard=%d , will be reset to 0)",soundcard);
      soundcard=0;
      return FALSE;
   }

//load up the samples
   for(n=0;n<6;n++)
   {
      strcpy(szWork,szSamples[n]);
      if(!strstr(szWork,".wav"))
         strcat(szWork,".wav");

      if(!readsample(szWork,n))
      {
         printf("Unable to load wav file %s",szWork);
         return FALSE;
      }
      lpWave[n]->dwLoopStart=lpWave[n]->dwLoopEnd = 0L;
   }


   AOpenVoices(NO_CHANNELS);

   for(n=0;n<NO_CHANNELS;n++)
   {
      ACreateAudioVoice(&hVoice[n]);
      ASetVoiceVolume(hVoice[n], 64);
      ASetVoicePanning(hVoice[n], 128);
   }

   ASetAudioMixerValue(AUDIO_MIXER_MASTER_VOLUME,(int)(((float)vol/15.0F)*256.0F));
   return TRUE;
}

int readsample(char *szFileName,int waveno)
{
   unsigned long int idx;
   unsigned long int odx;
   long int longin;
   long int temp32;
   short int shortin;
   short int bitres;
   int srate;
   pBYTE pWork;
   pBYTE buf;
   pBYTE pSave;
   long int len;
   FILE *fin;
//try simple case first
   fin=fopen(szFileName,"rb");
   if(fin)
   {
//got it, get the length
      fseek(fin,0,SEEK_END);
      len=ftell(fin);
      fseek(fin,0,SEEK_SET);
//allocate space for it
      buf=malloc(len);
      if(!buf)
      {
         fclose(fin);
         return FALSE;
      }
//read it all in
      fread(buf,1,len,fin);
      fclose(fin);
   }
   else
      return FALSE;


   pSave=buf;
   pWork=buf;

   if(memcmp(buf,"RIFF", 4)||memcmp(buf+8,"WAVE",4))
   {
      loginfo(1,"File not a valid WAV file\n");
      myfree(pSave);
      return FALSE;
   }
   buf+=12;

//format chunk
   if(memcmp(buf,"fmt",3))
   {
      loginfo(1,"File not a valid WAV file\n");
      myfree(pSave);
      return FALSE;
   }


   buf+=4;
//get size (should be 16)
   buf+=4;

//format (should be PCM)
   shortin=*((short int *)buf);

   if(shortin!=1)
   {
      loginfo(1,"File is not in PCM format\n");
      myfree(pSave);
      return FALSE;
   }
   buf+=2;
//channels
   shortin=*((short int *)buf);
   if(shortin!=1)
   {
      loginfo(1,"File is a stereo WAV file, not supported by VAntAGE\n");
      myfree(pSave);
      return FALSE;
   }
   buf+=2;
//sample rate
   longin=*((long int *)buf);
   buf+=4;
   buf+=6;

   bitres=*((short int *)buf);

   if (bitres!=8&&bitres!=16)
   {
      loginfo(1,"Only 16bit and 8bit WAV files supported (WAV is %dbit)",bitres);
      myfree(pSave);
      return FALSE;
   }


   len-=48;


        /* first allocate structure to hold the waveform object */
   lpWave[waveno]=(LPAUDIOWAVE)myalloc(sizeof(AUDIOWAVE));
   if(!lpWave[waveno])
   {
      loginfo(1,"Unable to allocate wave");
      myfree(pSave);
      return FALSE;
   }
   if(bitres==8)
      lpWave[waveno]->wFormat = AUDIO_FORMAT_8BITS | AUDIO_FORMAT_MONO;
   else
      lpWave[waveno]->wFormat = AUDIO_FORMAT_16BITS | AUDIO_FORMAT_MONO;
   lpWave[waveno]->nSampleRate = longin;
   lpWave[waveno]->dwLength = len;
   lpWave[waveno]->dwLoopStart = lpWave[waveno]->dwLoopEnd = 0;
   ACreateAudioData(lpWave[waveno]);
            /* copy the data into the waveform object */
   pWork+=46;
   memcpy(lpWave[waveno]->lpData, pWork, len);
   if(bitres==8)
   {
      for (temp32 = 0; temp32 < len; temp32++)
         lpWave[waveno]->lpData[temp32] ^= 0x80;
   }


   AWriteAudioData(lpWave[waveno],0L,len);
   myfree(pSave);
   return TRUE;
}

void clearsoundinterface(void)
{
   int n;
   BOOL stopped;
   int finished=FALSE;
   clock_t tickstart;
   clock_t ticker,tickwork;

   tickstart=clock();
   while(!finished)
   {
      finished=TRUE;
      for(n=0;n<NO_CHANNELS;n++)
      {
         AGetVoiceStatus(hVoice[n], &stopped);
         if(!stopped)
         {
            finished=FALSE;
            break;
         }
      }
      ticker=clock();
      tickwork=(ticker-tickstart)/clocks_per_second;
      if(tickwork>5)
         break;
      updatesound();
   }

   for(n=0;n<NO_CHANNELS;n++)
   {
      AStopVoice(hVoice[n]);
      ADestroyAudioVoice(hVoice[n]);
   }
   ACloseVoices();


   for(n=0;n<6;n++)
      AFreeWaveFile(lpWave[n]);

   ACloseAudio();
}

void soundinterface(int idx)
{
   int freq;

   if(soundcard)
   {
      ASetAudioMixerValue(AUDIO_MIXER_MASTER_VOLUME,(int)(((float)vol/15.0F)*256.0F));
      ASetVoiceVolume(hVoice[idx],(int)(((float)sampvol[idx]/15.0f)*64.0F));
      APlayVoice(hVoice[idx],lpWave[idx]);
      freq=(lpWave[idx]->nSampleRate*sampfreq[idx])/50;
      ASetVoiceFrequency(hVoice[idx],freq);
   }
}

void beep(int nobeeps)
{
	int n;
   for(n=0;n<nobeeps;n++)
	{
		if(SBPresent)
		{
//only beep using FM if sound (i.e. sample) interface is off
         if(soundcard)
            return;

			SetFreq(1,363);
			SetOct(1,3);
			KeyOn(1);
		}
		else
			sound(300);

		delay(100);

		if(SBPresent)
			KeyOff(1);
		else
			nosound();

		delay(100);
	}
}

void sounddelay(int secs)
{
   clock_t now;
   clock_t then;

   if(!soundcard)
      delay(secs*1000);
   else
   {
      then=clock();

      while(TRUE)
      {
         now=(clock()-then);
         if((now/clocks_per_second)>=secs)
            break;
         updatesound();
      }
   }
}

